import QtQuick 1.1
import Process 0.1

// Keypad.qml - Keypad control for entering numeric information.

import QtQuick 1.1

import "../"

Item {
    id: root

    signal valueEntered (string val)
    signal keyPress(string key)
    signal close()
    signal multipress(variant keylist)

    property variant keys_model: state === "lowercase" ? lowercase_keys : uppercase_keys
    property int length:20
    property string keymap: ""

    width: keypad_box.width
    height: childrenRect.height

    state: getState()

    function getState() {
        // Get initial state depending on language
        if (application.currentLanguage === "ar") {
            return "ArabicLowercase"
        }
        return "lowercase"
    }

    function setKeymap(keymapIn) {
        var keymap = "us"

        if (keymapIn) {
            keymap = keymapIn
        }
        console.log("Setting keymap to", keymap)
        setxkbmap.arguments = ["-layout", keymap]
        setxkbmap.start()
        root.keymap = keymap
    }

    function show() {
        root.visible = true
    }

    function hide() {
        root.visible = false
    }

    function keyFunction(val) {
        print("key function: " + val)
    }

    Connections {
        target: application
        onCurrentLanguageChanged: {
            // Reset state
            root.state = getState()
        }
        // Need to reset state any time keypad is reshown
        // Another keypad could have called setxkbmap and put this keypad out of sync w/ the global keymap
        onWindowShown: {
            console.log("window shown...")
            root.state = ""
            root.state = getState()
        }
    }

    Process {
        id: setxkbmap

        onReadyReadStandardError: {
            var output = new String(readAllStandardError())
            console.log(output)
        }
        onReadyReadStandardOutput: {
            var output = new String(readAllStandardOutput())
            console.log(output)
        }

        program: "/usr/bin/setxkbmap"
    }

    // stack top entry box on top of keypad

    // main keypad body
    Rectangle {
        id: keypad_box

        visible: true
        anchors.top: parent.top
        //anchors.topMargin: -37
        anchors.horizontalCenter: parent.horizontalCenter
        x: 10
        y: 90
        width: langButton.visible ? 890 : 830
        height: 300
        anchors.horizontalCenterOffset: 0
        smooth: true
        color: "gray"
        border.color: "black"
        border.width: 5
        radius: 20

        MouseArea {
            anchors.fill: parent
            onClicked: {} // Block clicks from affecting items underneath
        }

        ImageButton {
            id: xButton
            anchors { top: parent.top; right: parent.right; rightMargin: 2; topMargin: 2 }
            upimage: "Keypad/KeyPad_X_Large.png"
            downimage: upimage
            onClicked: {
                root.close()
            }
        }

        LanguageSelector {
            id: langButton

            visible: application.currentLanguage === "zh" || application.currentLanguage === "jp"
            anchors {
                bottom: parent.bottom
                bottomMargin: 0
                right: parent.right
                rightMargin: 1
            }
        }

        Column {
            id: keypad_column
            anchors { left: parent.left; top: parent.top }
            anchors.topMargin: 10
            anchors.leftMargin: 15
            anchors.rightMargin: 15
            width: 800
            spacing: 0
            Flow {
                id: keygrid
                width: parent.width
                spacing: 5
                anchors.horizontalCenter: parent.horizontalCenter

                Repeater {
                    model: keys_model
                    delegate: keypad_button
                }
            }
        }
    }

    // delegate for keypad button
    Component {
        id: keypad_button
        ImageButton {
            upimage: "Keypad/" + model.buttonimage + "_UP.png"
            downimage: "Keypad/"+ model.buttonimage + "_DOWN.png"
            icon: image ? image : ""
            label: size ? model.label : ""
            label_color: "black"
            label_visible: (application.currentLanguage !== "en" && value === "KP_Enter") ? false : true
            fontsize: size
            label_x_offset: x_off
            label_y_offset: y_off
            scale: 0.9
            enabled: true
            onClicked: {
                if(func) {
                    print(func)
                    eval(func)
                }
                else
                    keyPress(value)
            }
        }
    }

    states: [
        // States for Arabic
        State {
            name: "ArabicUppercase"
            PropertyChanges { target: root ; keys_model: arabic_uppercase }
            PropertyChanges { target: keypad_box; width: 980 }
            PropertyChanges { target: keypad_column; width: 950 }
            StateChangeScript { script: root.setKeymap("ar") }
        },
        State {
            name: "ArabicLowercase"
            PropertyChanges { target: root; keys_model: arabic_lowercase }
            PropertyChanges { target: keypad_box; width: 980 }
            PropertyChanges { target: keypad_column; width: 950 }
            StateChangeScript { script: root.setKeymap("ar") }
        },
        State {
            name: "ArabicAlpha"
            PropertyChanges { target: root; keys_model: ar_lowercase_keys }
            PropertyChanges { target: keypad_box; width: 830 }
            PropertyChanges { target: keypad_column; width: 800 }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "ArabicAlphaUpper"
            PropertyChanges { target: root; keys_model: ar_uppercase_keys }
            PropertyChanges { target: keypad_box; width: 830 }
            PropertyChanges { target: keypad_column; width: 800 }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "ArabicNumeric"
            PropertyChanges { target: root; keys_model: ar_number_keys }
            PropertyChanges { target: keypad_box; width: 830 }
            PropertyChanges { target: keypad_column; width: 800 }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "ArabicSymbols"
            PropertyChanges { target: root; keys_model: ar_symbol_keys }
            PropertyChanges { target: keypad_box; width: 830 }
            PropertyChanges { target: keypad_column; width: 800 }
            StateChangeScript { script: root.setKeymap("us") }
        },


        State {
            name: "uppercase"
            PropertyChanges { target: root ; keys_model: uppercase_keys }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "lowercase"
            PropertyChanges { target: root ; keys_model: lowercase_keys }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "numbers"
            PropertyChanges { target: root ; keys_model: number_keys }
            StateChangeScript { script: root.setKeymap("us") }
        },
        State {
            name: "symbols"
            PropertyChanges { target: root ; keys_model: symbol_keys }
            StateChangeScript { script: root.setKeymap("us") }
        }
    ]

    // model of keypad keys
    // The `value` column comes from X11/keysymdef.h as this is what must be passed to xdotool
    ListModel {
        id: uppercase_keys

        ListElement { label: "Q" ; value: "Q"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "W"; value: "W"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "E"; value: "E"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "R"; value: "R"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "T"; value: "T"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "Y"; value: "Y"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "U"; value: "U"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "I"; value: "I"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "O"; value: "O"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "P"; value: "P"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "A" ; value: "A"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "S"; value: "S"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "D"; value: "D"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "F"; value: "F"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "G"; value: "G"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "H"; value: "H"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "J"; value: "J"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "K"; value: "K"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "L"; value: "L"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "" ; value: ""; buttonimage: "keyboard_single" ; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='lowercase'"}
        ListElement { label: "Z"; value: "Z"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "X"; value: "X"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "C"; value: "C"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "V"; value: "V"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "B"; value: "B"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "N"; value: "N"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "M"; value: "M"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}
        ListElement { label: "?123"; value: ""; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='numbers'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }

    ListModel {
        id: lowercase_keys

        ListElement { label: "q"; value: "q"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "w"; value: "w"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "e"; value: "e"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "r"; value: "r"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "t"; value: "t"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "y"; value: "y"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "u"; value: "u"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "i"; value: "i"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "o"; value: "o"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "p"; value: "p"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "a"; value: "a"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "s"; value: "s"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "d"; value: "d"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "f"; value: "f"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "g"; value: "g"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "h"; value: "h"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "j"; value: "j"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "k"; value: "k"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "l"; value: "l"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: ""; value: ""; buttonimage: "keyboard_single" ; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='uppercase'"}
        ListElement { label: "z"; value: "z"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "x"; value: "x"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "c"; value: "c"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "v"; value: "v"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "b"; value: "b"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "n"; value: "n"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "m"; value: "m"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "?123"; value: "?123"; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='numbers'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }

    // Based on IBM 101-key Arabic keyboard
    // See <ftp://ftp.software.ibm.com/software/globalization/keyboards/KBD470.pdf>
    ListModel {
        id: arabic_lowercase

        ListElement { label: "\uFEBF"; value: "Arabic_dad"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEBB"; value: "Arabic_sad"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE9B"; value: "Arabic_theh"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFED7"; value: "Arabic_qaf"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFED3"; value: "Arabic_feh"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFECF"; value: "Arabic_ghain"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFECB"; value: "Arabic_ain"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEEB"; value: "Arabic_heh"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEA7"; value: "Arabic_khah"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEA3"; value: "Arabic_hah"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE9F"; value: "Arabic_jeem"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEA9"; value: "Arabic_dal"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "\uFEB7"; value: "Arabic_sheen"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEB3"; value: "Arabic_seen"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEF3"; value: "Arabic_yeh"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE91"; value: "Arabic_beh"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEDF"; value: "Arabic_lam"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE8D"; value: "Arabic_alef"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE97"; value: "Arabic_teh"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEE7"; value: "Arabic_noon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEE3"; value: "Arabic_meem"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEDB"; value: "Arabic_kaf"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEC3"; value: "Arabic_tah"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: ""; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0; func: ""} // empty key

        ListElement { label: ""; value: ""; buttonimage: "keyboard_single" ; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicUppercase'"}
        ListElement { label: "\uFE8B"; value: "Arabic_hamzaonyeh"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE80"; value: "Arabic_hamza"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE85"; value: "Arabic_hamzaonwaw"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEAD"; value: "Arabic_ra"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEFB"; value: 0x100fefb; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0} // TODO should this be 0x100fefb? - see "dist/symbols/ara"
        ListElement { label: "\uFEEF"; value: "Arabic_alefmaksura"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFE93"; value: "Arabic_tehmarbuta"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEED"; value: "Arabic_waw"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEAF"; value: "Arabic_zain"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\uFEC7"; value: "Arabic_zah"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlpha'"}
        ListElement { label: "?123"; value: "?123"; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }

    ListModel {
        id: arabic_uppercase
        ListElement { label: "\u064E"; value: "Arabic_fatha"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064B"; value: "Arabic_fathatan"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064F"; value: "Arabic_damma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064C"; value: "Arabic_dammatan"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: "??"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0625"; value: "Arabic_hamzaunderalef"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0060"; value: "grave"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u00F7"; value: "division"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u00D7"; value: "multiply"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u061B"; value: "Arabic_semicolon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        /*Less Than*/ListElement { label: "<"; value: "Arabic_semicolon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        /*Greater Than*/ListElement { label: ">"; value: "Arabic_semicolon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "\u0650"; value: "Arabic_kasra"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u064D"; value: "Arabic_kasratan"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "]"; value: "bracketright"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "["; value: "bracketleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: "0x100fef7"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0623"; value: "Arabic_hamzaonalef"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0640"; value: "Arabic_tatweel"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u060C"; value: "Arabic_comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ":"; value: "colon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\""; value: "quotedbl"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: ""; value: ""; buttonimage: "keyboard_single" ; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "~"; value: "asciitilde"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0652"; value: "Arabic_sukun"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "}"; value: "braceright"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "{"; value: "braceleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: " "; value: "0x100fef5"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u0622"; value: "Arabic_maddaonalef"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "'"; value: "apostrophe"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\u061F"; value: "Arabic_question_mark"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlpha'"}
        ListElement { label: "?123"; value: "?123"; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_lowercase_keys

        ListElement { label: "q"; value: "q"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "w"; value: "w"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "e"; value: "e"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "r"; value: "r"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "t"; value: "t"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "y"; value: "y"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "u"; value: "u"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "i"; value: "i"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "o"; value: "o"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "p"; value: "p"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "a"; value: "a"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "s"; value: "s"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "d"; value: "d"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "f"; value: "f"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "g"; value: "g"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "h"; value: "h"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "j"; value: "j"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "k"; value: "k"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "l"; value: "l"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: ""; value: ""; buttonimage: "keyboard_single" ; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlphaUpper'"}
        ListElement { label: "z"; value: "z"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "x"; value: "x"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "c"; value: "c"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "v"; value: "v"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "b"; value: "b"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "n"; value: "n"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "m"; value: "m"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; buttonimage: "keyboard_singlehalf"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "?#"; value: "?123"; buttonimage: "keyboard_singlehalf"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER"; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_singlehalf"; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_uppercase_keys

        ListElement { label: "Q" ; value: "Q"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "W"; value: "W"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "E"; value: "E"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "R"; value: "R"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "T"; value: "T"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "Y"; value: "Y"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "U"; value: "U"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "I"; value: "I"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "O"; value: "O"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "P"; value: "P"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "A" ; value: "A"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "S"; value: "S"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "D"; value: "D"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "F"; value: "F"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "G"; value: "G"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "H"; value: "H"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "J"; value: "J"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "K"; value: "K"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "L"; value: "L"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "" ; value: ""; buttonimage: "keyboard_single" ; image: "Keypad/keyboard_SHIFT.png"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicAlpha'"}
        ListElement { label: "Z"; value: "Z"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "X"; value: "X"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "C"; value: "C"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "V"; value: "V"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "B"; value: "B"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "N"; value: "N"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "M"; value: "M"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; buttonimage: "keyboard_singlehalf"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "?#"; value: ""; buttonimage: "keyboard_singlehalf"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter"; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_singlehalf"; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_number_keys

        ListElement { label: "1" ;buttonimage: "keyboard_single"; value: "1" ; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "2"; value: "2"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "3"; value: "3"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "4"; value: "4"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "5"; value: "5"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "6"; value: "6"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "7"; value: "7"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "8"; value: "8"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "9"; value: "9"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "0"; value: "0"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "-" ; value: "minus"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ":"; value: "colon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "("; value: "parenleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ")"; value: "parenright"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: 'quotedbl' ;buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "#+="; value: "" ; buttonimage: "keyboard_single";  size: 30; x_off: 0; y_off: 0 ; func: "root.state='ArabicSymbols'"}
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; buttonimage: "keyboard_singlehalf"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ;image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }
    ListModel {
        id: ar_symbol_keys

        ListElement { label: "]"; value: "bracketright"; buttonimage: "keyboard_single";  size: 45; x_off: 0; y_off: 0}
        ListElement { label: "]"; value: "bracketleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "{"; value: "braceleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "}"; value: "braceright"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "%"; value: "percent"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "^"; value: "acute"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "+"; value: "plus"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "_" ;buttonimage: "keyboard_single"; value: "underscore" ; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\\"; value: "backslash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "|"; value: "bar"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "~"; value: "asciitilde"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<"; value: "less"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ">"; value: "greater"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: "quotedbl" ;buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "123" ; value: "" ; buttonimage: "keyboard_single";  size: 30; x_off: 0; y_off: 0 ; func: "root.state='ArabicNumeric'"}
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "AR"; value: ""; buttonimage: "keyboard_singlehalf"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='ArabicLowercase'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }

    // Default - English
    ListModel {
        id: number_keys

        ListElement { label: "1" ;buttonimage: "keyboard_single"; value: "1" ; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "2"; value: "2"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "3"; value: "3"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "4"; value: "4"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "5"; value: "5"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "6"; value: "6"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "7"; value: "7"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "8"; value: "8"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "9"; value: "9"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "0"; value: "0"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "-" ; value: "minus"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ":"; value: "colon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ";"; value: "semicolon"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "("; value: "parenleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ")"; value: "parenright"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: 'quotedbl' ;buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "#+="; value: "" ; buttonimage: "keyboard_single";  size: 30; x_off: 0; y_off: 0 ; func: "root.state='symbols'"}
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='lowercase'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ;image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }

    ListModel {
        id: symbol_keys

        ListElement { label: "]"; value: "bracketright"; buttonimage: "keyboard_single";  size: 45; x_off: 0; y_off: 0}
        ListElement { label: "]"; value: "bracketleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "{"; value: "braceleft"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "}"; value: "braceright"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "%"; value: "percent"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "^"; value: "acute"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "+"; value: "plus"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "_" ;buttonimage: "keyboard_single"; value: "underscore" ; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "\\"; value: "backslash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "|"; value: "bar"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "~"; value: "asciitilde"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<"; value: "less"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ">"; value: "greater"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "$"; value: "dollar"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "&"; value: "ampersand"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: '"'; value: "quotedbl" ;buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}

        ListElement { label: "123" ; value: "" ; buttonimage: "keyboard_single";  size: 30; x_off: 0; y_off: 0 ; func: "root.state='numbers'"}
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ","; value: "comma"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "?"; value: "question"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "!"; value: "exclam"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "#"; value: "numbersign"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "*"; value: "asterisk"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "="; value: "equal"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "@"; value: "at"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "<<"; value: "BackSpace";  image: "Keypad/keyboard_BACKSPACE.png"; buttonimage: "keyboard_single"; size: 0; x_off: 0; y_off: 0}

        ListElement { label: "ABC"; value: ""; buttonimage: "keyboard_double"; size: 45; x_off: 0; y_off: 0 ; func: "root.state='lowercase'"}
        ListElement { label: "SPACE"; value: "space"; buttonimage: "keyboard_triple"; size: 45; x_off: 0; y_off: 0 }
        ListElement { label: "."; value: "period"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: "/"; value: "slash"; buttonimage: "keyboard_single"; size: 45; x_off: 0; y_off: 0}
        ListElement { label: ".com"; value: ".com"; buttonimage: "keyboard_single"; size: 16; x_off: 0; y_off: 0; func: "root.multipress(['period', 'c', 'o', 'm'])" }
        ListElement { label: "ENTER" ; value: "KP_Enter" ; image: "Keypad/keypad_returnicon.png"; buttonimage: "keyboard_double"; size: 14; x_off: 0; y_off: -10}
    }


}
